"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const run_1 = require("../../base/run");
const docker_run_1 = require("../../core/docker.run");
const conf_1 = require("../../services/conf");
const util_1 = require("../../util");
class DockerRun extends run_1.default {
    async preInit() {
        await super.preInit();
        const { flags } = this.parse(DockerRun);
        this.extras = flags.extras;
    }
    async preRun() {
        const core = new docker_run_1.default({
            watch: this.watch,
        });
        await core.start();
    }
    async run() {
        if (!this.cmd) {
            return;
        }
        process.env.PATH = `${process.env.PATH}:${conf_1.default.cwd}/node_modules/.bin`;
        let args = ['run', `${this.cmd}`];
        if (this.extras) {
            args.push('--');
            let extras = [];
            if (this.cmd === 'lint') {
                extras = this.extras.split(' ').map(item => {
                    // fix path to relative to .rde path
                    if (/^(app|template)\/.*/.test(item)) {
                        return path.join('..', item);
                    }
                    return item;
                });
            }
            else {
                extras = this.extras.split(' ');
            }
            args = args.concat(extras);
        }
        await util_1.default.asyncSpawn('npm', args, {
            cwd: conf_1.default.isIntegrate ? conf_1.default.integrateDir : conf_1.default.runtimeDir,
            env: process.env
        });
        process.exit(0);
    }
}
DockerRun.description = 'run script inside docker container';
DockerRun.examples = [
    '$ rde docker:run <cmd>',
];
DockerRun.args = [...run_1.default.args];
DockerRun.flags = Object.assign({}, run_1.default.flags);
exports.default = DockerRun;
