"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const cleanStack = require("clean-stack");
const fs = require("fs");
const ip = require("ip");
const path = require("path");
const rax = require("retry-axios");
const conf_1 = require("./conf");
// init request
const request = axios_1.default.create();
request.defaults.raxConfig = { instance: request };
request.defaults.headers.common['X-SHA1-APPKEY'] = '53ce9ed1f8631045ca098a0949b0278e72bcc00f'; // APP_KEY SHA1加密
request.defaults.headers.common['X-CLIENT-IP'] = ip.address();
rax.attach(request);
// Hubble config
const serverUrl = 'https://hubble.netease.com/track/s/';
const appKey = 'MA-AC3D-F6D2C4E407FA';
const isDebug = false;
const logTrack = msg => {
    isDebug && console.log(msg); // tslint:disable-line
};
exports.uploadHubble = (info = {}) => {
    const name = path.basename(conf_1.default.cwd);
    const { appConfPath, rdcConfPath } = conf_1.default;
    let tag = 'RDC';
    if (fs.existsSync(appConfPath)) {
        const appConf = conf_1.default.getAppConf();
        tag = appConf.container.name;
    }
    else if (fs.existsSync(rdcConfPath)) {
        const rdcConf = conf_1.default.getRdcConf('.');
        tag = rdcConf.docker.tag;
    }
    try {
        const params = {
            userId: name || tag.split(':')[0],
            dataType: 'ie',
            sdkType: 'server',
            eventId: 'RdeTrack',
            time: (new Date).getTime(),
            appKey,
            attributes: Object.assign({}, info, { version: tag, project: name })
        };
        const data = Buffer.from(JSON.stringify(params)).toString('base64');
        request({
            method: 'get',
            url: serverUrl,
            params: { data },
            raxConfig: {
                onRetryAttempt: err => {
                    const cfg = rax.getConfig(err);
                    logTrack(`Retry attempt #${cfg.currentRetryAttempt}`);
                }
            }
        });
    }
    catch (_e) {
        if (_e) { }
    }
};
function TStart(target, key, descriptor) {
    return Object.assign({}, descriptor, { async value(...params) {
            try {
                logTrack(`\n开始执行----->: ${key} in ${target.constructor.name}\n`);
                exports.uploadHubble({ executor: `${key} in ${target.constructor.name}` });
                await descriptor.value.apply(this, params);
            }
            catch (err) {
                throw Error(err);
            }
        } });
}
exports.TStart = TStart;
exports.TError = (params = {}) => {
    return (target, key, descriptor) => {
        // if(descriptor === undefined) {
        //     descriptor = Object.getOwnPropertyDescriptor(target, key);
        // }
        const oldValue = descriptor.value;
        descriptor.value = async function (...args) {
            try {
                const result = await oldValue.apply(this, args);
                logTrack(`完成执行${key} in ${target.constructor.name}----->: ${result || '无返回结果'}`);
                await exports.uploadHubble({ executor: `${key} in ${target.constructor.name}`, result });
                return result;
            }
            catch (err) {
                logTrack(`\n捕获错误----->: ${key} in ${target.constructor.name}, ${args}`);
                logTrack(`\n步骤${key} in ${target.constructor.name}发生错误: ${cleanStack(err.stack)},
        \n收集上报的额外参数:${JSON.stringify(params)}\n`);
                await exports.uploadHubble({ executor: `${key} in ${target.constructor.name}`, errors: String(err) });
                throw Error(err);
            }
        };
        return descriptor;
    };
};
