"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar = require("chokidar");
const Events = require("events");
const fs = require("fs");
const path = require("path");
const util_1 = require("../util");
const conf_1 = require("./conf");
const logger_1 = require("./logger");
class Watcher extends Events {
    constructor(mappings = [], copy) {
        super();
        this.mappings = [];
        this.mappings = mappings;
        this.copy = copy || util_1.default.copy;
    }
    start() {
        const watchFiles = this.mappings.map(item => path.resolve(conf_1.default.cwd, item.from));
        const watcher = chokidar.watch(watchFiles, {
            ignored: /(\.git)|(node_modules)/,
            ignoreInitial: true,
            usePolling: true,
            interval: 300,
            binaryInterval: 300
        });
        watcher.on('add', path => this.handler('add', path))
            .on('change', path => this.handler('change', path))
            .on('unlink', path => this.handler('unlink', path))
            .on('addDir', path => this.handler('addDir', path))
            .on('unlinkDir', path => this.handler('unlinkDir', path));
    }
    async handler(type, filePath) {
        const { cwd } = conf_1.default;
        const { resolve, relative } = path;
        const relativeToAppPath = relative(cwd, filePath);
        const mapping = this.mappings.find(item => relativeToAppPath.includes(item.from));
        // 截取从from 到 变动文件的 路径
        const relativeToFromPath = relative(resolve(cwd, mapping.from), filePath);
        // 将相对路径 加在 to 后面，拼成 完整的目标路径
        const destPath = resolve(conf_1.default.integrateDir, mapping.to, relativeToFromPath);
        if (['add', 'change', 'addDir'].includes(type)) {
            await this.copy(filePath, destPath, mapping);
        }
        if (type === 'unlink') {
            fs.unlinkSync(destPath);
        }
        if (type === 'unlinkDir') {
            try {
                fs.rmdirSync(destPath);
            }
            catch (e) {
                if (e) {
                    logger_1.logger.warn(`Directory ${destPath} is not empty, delete filed!`);
                }
            }
        }
        logger_1.logger.info(`Updated: ${filePath}`);
        this.emit('update');
    }
}
exports.default = Watcher;
